@@ -9,8 +9,8 @@ from ckan.lib.navl.dictization_functions import DataError
 from ckan.lib.redis import connect_to_redis
 from ckan.logic import side_effect_free
 from ckan.plugins import toolkit
-
-# from ckan.types import Context, DataDict
+from ckan.types import Context, DataDict
+from requests import get as get_url
 from sqlalchemy.exc import InternalError as SQLAlchemyError
 
 from ckanext.passwordless_api import util
@@ -20,14 +20,15 @@ log = logging.getLogger(__name__)
 
 
 def request_reset_key(
-    context,  #: Context,
-    data_dict,  #: DataDict
+    context: Context,
+    data_dict: DataDict,
 ):
-    """
-    Request a reset key (login token) to be sent by email.
+    """Request a reset key (login token) to be sent by email.
 
     Args:
-        data_dict.email (str): Email of user.
+        context (Context): CKAN context, including user.
+        data_dict (DataDict):
+            - email (str): Email of user.
 
     Returns:
         dict: {message: 'success'}
@@ -71,10 +72,12 @@ def request_reset_key(
 
         except mailer.MailerException as e:
             log.error(f"Could not send token link: {str(e)}")
-            raise mailer.MailerException(f"Could not send token link by mail: {str(e)}")
+            raise mailer.MailerException from e(
+                f"Could not send token link by mail: {str(e)}"
+            )
 
     else:
-        raise toolkit.ValidationError(
+        raise toolkit.ValidationError from None(
             {"user": "cannot retrieve or create user with given email"}
         )
 
@@ -101,23 +104,26 @@ def _create_user(email):
         exception_message = f"{error}"
         log.error(f"Failed to create user: {error}")
         if exception_message.find("quota") >= 0:
-            raise DataError("Error creating a new user, daily new user quota exceeded")
+            raise DataError from error(
+                "Error creating a new user, daily new user quota exceeded"
+            )
         else:
-            raise DataError("Internal error creating a new user")
+            raise DataError from error("Internal error creating a new user")
 
     return user
 
 
 def request_api_token(
-    context,  #: Context,
-    data_dict,  #: DataDict
+    context: Context,
+    data_dict: DataDict,
 ):
-    """
-    Get API token for a user, if reset key valid.
+    """Get API token for a user, if reset key valid.
 
     Args:
-        data_dict.email (str): Email of user.
-        data_dict.key (str): Reset key of user (from email after request_reset_key).
+        context (Context): CKAN context, including user.
+        data_dict (DataDict):
+            - email (str): Email of user.
+            - key (str): Reset key of user (from email after request_reset_key).
 
     Returns:
         dict: CKAN API token for user {'token': token_value}.
@@ -171,103 +177,63 @@ def request_api_token(
 
 
 def request_api_token_azure_ad(
-    context,  #: Context,
-    data_dict,  #: DataDict
+    context: Context,
+    data_dict: DataDict,
 ):
-    """
-    Get API token for a user, if valid Azure AD token provided.
+    """Get API token for a user, if valid Azure AD token provided.
 
-    To be called from redirectUri page on frontend.
-    Must be configured in the plugin settings:
-        - azure_ad_tenant_id
-        - azure_ad_client_id
-        - azure_ad_client_secret
+    To be called from a custom frontend, using a Azure AD login flow.
 
     Args:
-        data_dict.email (str): Email of user.
-        data_dict.auth_code (str): Token generated by authorization code
-            flow in Azure AD frontend login.
-        data_dict.redirect_uri (str): The redirect uri used in the frontend Azure AD
-            login flow (must match).
-        data_dict.state (str): The state generated from the frontend Azure AD
-            login flow (must match).
+        context (Context): CKAN context, including user.
+        data_dict (DataDict):
+            - email (str): Email of user.
+            - token (token): Token generated by Azure AD from frontend.
 
     Returns:
         dict: CKAN API token for user {'token': token_value}.
     """
-    log.debug(f"Requesting API token using Azure AD auth code with params: {data_dict}")
+    log.debug(f"Requesting API token using Azure AD token params: {data_dict}")
 
     # Check if parameters are present
     if not (email := data_dict.get("email")):
         raise toolkit.ValidationError({"email": "missing email"})
-    if not (auth_code := data_dict.get("auth_code")):
-        raise toolkit.ValidationError({"auth_code": "missing authorization code"})
-    if not (redirect_uri := data_dict.get("redirect_uri")):
-        raise toolkit.ValidationError({"redirect_uri": "missing redirect uri"})
-    if not (state := data_dict.get("state")):
-        raise toolkit.ValidationError({"state": "missing state param from login flow"})
-
-    log.debug("Attempting to import requests package")
-    try:
-        from requests import post
-    except ImportError as e:
-        log.error(e)
-        raise ImportError(
-            "This endpoint requires the `requests` dependency to be installed. "
-            "Try installing with `pip install requests`."
-        )
-
-    azure_ad_tenant_id = config.get("passwordless_api.azure_ad_tenant_id", None)
-    azure_ad_client_id = config.get("passwordless_api.azure_ad_client_id", None)
-    azure_ad_client_secret = config.get("passwordless_api.azure_ad_client_secret", None)
-    if None in (azure_ad_tenant_id, azure_ad_client_id, azure_ad_client_secret):
-        err_str = (
-            "passwordless_api.azure_ad_xxx settings are required to use this endpoint"
-        )
-        log.error(err_str)
-        raise toolkit.ObjectNotFound(err_str)
-
-    # Validate Azure AD token
-    try:
-        url = (
-            "https://login.microsoftonline.com/"
-            f"{azure_ad_tenant_id}/oauth2/v2.0/token"
-        )
-        params = {
-            "client_id": azure_ad_client_id,
-            "grant_type": "authorization_code",
-            "code": auth_code,
-            "state": state,
-            "redirect_uri": redirect_uri,
-            "client_secret": azure_ad_client_secret,
-        }
-        log.debug(f"Requesting token from url {url} with params {params}")
-        response = post(url, data=params)
-        log.debug(response.content)
-
-        access_token = response.json().get("access_token", None)
-        if not access_token:
-            raise toolkit.NotAuthorized("the login attempt was unsuccessful")
-
-    except Exception as e:
-        log.error(e)
-        raise toolkit.ValidationError({"auth_code": "unknown error during validation"})
-    # if not payload:
-    #     log.error("Azure AD JWT validation failed")
-    #     raise toolkit.NotAuthorized({"message": "Azure AD JWT validation failed"})
+    if not (token := data_dict.get("token")):
+        raise toolkit.ValidationError({"token": "missing token"})
 
     # Check email valid
     email = email.lower()
     if not util.email_is_valid(email):
         raise toolkit.ValidationError({"email": "invalid email"})
+
+    # Validate Azure AD token
+    try:
+        log.debug("Verifying JWT token")
+        response = get_url(
+            "https://graph.microsoft.com/v1.0/me", headers={"Authorization": token}
+        ).json()
+        log.debug(f"Returned response for verification: {response}")
+
+        azure_ad_email = response.get("mail", None)
+        if azure_ad_email != email:
+            raise toolkit.NotAuthorized from None(
+                "email from azure verification does not match email provided "
+                "from frontend app. verification failed"
+            )
+
+    except Exception as e:
+        log.error(e)
+        raise toolkit.ValidationError from e(
+            {"token": "unknown error while validating token with azure"}
+        )
+
     # Check user exists, else create new user
     if not (user := util.get_user_from_email(email)):
         _create_user(email)
         log.debug(f"Created user {str(email)}")
         user = util.get_user_from_email(email)
-
+    # Get user id from database model
     user_id = user.id
-    log.debug(f"User id: {user_id} {auth_code}")
 
     # delete attempts from Redis
     log.debug(f"Redis: reset attempts for {email}")
@@ -281,17 +247,18 @@ def request_api_token_azure_ad(
 
 @side_effect_free
 def revoke_api_token_no_auth(
-    context,  #: Context,
-    data_dict,  #: DataDict
+    context: Context,
+    data_dict: DataDict,
 ):
-    """
-    Revoke API token for a user, without requiring auth.
+    """Revoke API token for a user, without requiring auth.
 
     Useful for when user API token has expired already, but they wish to check
     if revokation is possible (i.e. during logout - revoke if possible).
 
     Args:
-        data_dict.token (str, optional): Value of API token to revoke.
+        context (Context): CKAN context, including user.
+        data_dict (DataDict):
+            - token (str, optional): Value of API token to revoke.
 
     Returns:
         dict: {message: 'success'}
@@ -340,11 +307,10 @@ def revoke_api_token_no_auth(
 
 @side_effect_free
 def get_current_user_and_renew_api_token(
-    context,  #: Context,
-    data_dict,  #: DataDict
+    context: Context,
+    data_dict: DataDict,
 ):
-    """
-    Return CKAN user and renew API token.
+    """Return CKAN user and renew API token.
 
     Uses the user_show core action to return the user, and renews the main API token.
 
